package hu.kozadat.search;

import hu.kozadat.search.persistence.core.RdbmsDataProviderOperations;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.ApplicationContext;

import java.text.Normalizer;
import java.util.Map;
import java.util.StringTokenizer;

/**
 * User: codex
 */
public class Utils {

    private static final Logger LOGGER = LoggerFactory.getLogger(Utils.class);

    private static final int RADIX_HEXA = 16;
    private static final int CHAR_SPACE = ' ';
    private static final int CHAR_PLUS = '+';
    private static final int CHAR_PERCENT = '%';
    private static final int TESTPATTERN_CONTINUATIONBYTE = 0xc0;
    private static final int PATTERN_CONTINUATIONBYTE = 0x80;
    private static final int MASK_PRECIOUSBITS_CONTINUATIONBYTE = 0x3f;
    private static final int TESTPATTERN_1BYTE = 0x80;
    private static final int PATTERN_1BYTE = 0x00;
    private static final int TESTPATTERN_2BYTES = 0xe0;
    private static final int PATTERN_2BYTES = 0xc0;
    private static final int MASK_PRECIOUSBITS_HIGHESTBYTE_2BYTES = 0x1f;
    private static final int TESTPATTERN_3BYTES = 0xf0;
    private static final int PATTERN_3BYTES = 0xe0;
    private static final int MASK_PRECIOUSBITS_HIGHESTBYTE_3BYTES = 0x0f;
    private static final int TESTPATTERN_4BYTES = 0xf8;
    private static final int PATTERN_4BYTES = 0xf0;
    private static final int MASK_PRECIOUSBITS_HIGHESTBYTE_4BYTES = 0x07;
    private static final int TESTPATTERN_5BYTES = 0xfc;
    private static final int PATTERN_5BYTES = 0xf8;
    private static final int MASK_PRECIOUSBITS_HIGHESTBYTE_5BYTES = 0x03;
    private static final int MASK_PRECIOUSBITS_HIGHESTBYTE_MOREBYTES = 0x01;

    public static final String DELIMITER_PLACEHOLDER = "%";

    /**
     * todo: write this
     */
    private static String getValue(String actToken, Map parameters, boolean parameterExists, String defValue) {
        if (actToken.length() > 0) {
            if (parameterExists && parameters.containsKey(actToken)) {
                String value = (String) parameters.get(actToken);
                return (value == null) ? "" : value;
            } else {
                return defValue;
            }
        } else {
            return DELIMITER_PLACEHOLDER;
        }
    }

    /**
     * todo: write this
     */
    public static String replacePlaceHolders(String rawString, Map parameters, String defValue) {
        if (rawString != null) {
            boolean parameterExists = (parameters != null) && !parameters.isEmpty();
            /*
                * iterate through all tokens of rawString (including DELIMITER_PLACEHOLDERs)
                * and process them (while in text mode, everything is copied to the result,
                * in non-text mode recognized placeholders are replaced by their values)
                */
            StringTokenizer tokenizer = new StringTokenizer(rawString, DELIMITER_PLACEHOLDER, true);
            StringBuffer res = new StringBuffer();
            StringBuffer buf = new StringBuffer();
            boolean textMode = true;
            while (tokenizer.hasMoreTokens()) {
                String actToken = tokenizer.nextToken();
                if (DELIMITER_PLACEHOLDER.equals(actToken)) {
                    if (textMode) {
                        buf.append(actToken);
                    } else {
                        res.append(getValue(buf.substring(1).toString(), parameters, parameterExists, defValue));
                        buf.setLength(0);
                    }
                    textMode = !textMode;
                } else if (textMode) {
                    res.append(actToken);
                } else {
                    buf.append(actToken);
                }
            }
            if (buf.length() > 0) {
                //if rawString ends with an unended non-text mode token, just copy it to the result
                res.append(buf);
            }
            return res.toString();
        } else {
            //return null for null rawString
            return null;
        }
    }

    public static final long WRAP_FIELD_USE_WHITESPACE = 0x0001;
    public static final long WRAP_FIELD_USE_SEPARATOR = 0x0002;
    public static final long WRAP_FIELD_USE_BLOCK = 0x0004;

    public static String makeStringWrap(String str, int rowSize) {
        return makeStringWrap(str, rowSize, WRAP_FIELD_USE_BLOCK + WRAP_FIELD_USE_SEPARATOR + WRAP_FIELD_USE_WHITESPACE);
    }

    public static String makeStringWrap(String str, int rowSize, long options) {
        final char separators[] = {'.', ',', ';', '!', '?', ':'};

        StringBuffer res = new StringBuffer();

        int remainingSize = str.length();
        int cursor = 0;
        while (remainingSize > 0) {
            //-- get the next block
            int blockSize = str.length() - cursor > rowSize ? rowSize : str.length() - cursor;
            String block = str.substring(cursor, cursor + blockSize - 1);

            int lastIdx = -1;

            //-- try to find the last whitespace
            if ((options & WRAP_FIELD_USE_WHITESPACE) > 0) {
                lastIdx = block.lastIndexOf(' ');

                if (lastIdx != -1) {
                    String subBlock = block.substring(0, lastIdx);
                    res.append(subBlock);
                    cursor += (lastIdx + 1);
                    remainingSize -= (lastIdx + 1);
                    continue;
                }
            }

            //-- try to find some kind a of separator
            if ((options & WRAP_FIELD_USE_SEPARATOR) > 0) {
                int bestIdx = -1;
                for (char ch : separators) {
                    lastIdx = block.lastIndexOf(ch);
                    if (lastIdx != -1 && lastIdx > bestIdx)
                        bestIdx = lastIdx;
                }

                if (bestIdx != -1) {
                    String subBlock = block.substring(0, bestIdx);
                    res.append(subBlock);
                    res.append(" ");
                    cursor += (bestIdx + 1);
                    remainingSize -= (bestIdx + 1);
                    continue;
                }
            }

            //-- create a space at the end of the block
            if ((options & WRAP_FIELD_USE_BLOCK) > 0) {
                res.append(block.substring(0, blockSize - 1));
                res.append(" ");
                cursor += blockSize;
                remainingSize -= blockSize;
                continue;
            }

            //-- We just add the block, that's it
            res.append(block.substring(0, blockSize - 1));
            cursor += blockSize;
            remainingSize -= blockSize;
        }


        return res.toString();
    }

    public static String unescape(String s) {
        if (s != null) {
            int len = s.length();
            StringBuffer res = new StringBuffer(len);
            int fwd = 0;
            int more = -1;
            int pos = 0;
            while (pos < len) {
                int ch = s.charAt(pos);
                pos++;
                int b;
                switch (ch) {
                    case CHAR_PERCENT:
                        b = Integer.parseInt(s.substring(pos, pos + 2), RADIX_HEXA);
                        pos += 2;
//						ch = s.charAt(pos);
//						pos++;
//						int hb = (Character.isDigit((char) ch) ? ch - '0' : 10 + Character.toLowerCase((char) ch) - 'a') & 0xF;
//						ch = s.charAt(pos);
//						pos++;
//						int lb = (Character.isDigit((char) ch) ? ch - '0' : 10 + Character.toLowerCase((char) ch) - 'a') & 0xF;
//						b = (hb << 4) | lb;
                        break;
                    case CHAR_PLUS:
                        b = CHAR_SPACE;
                        break;
                    default:
                        b = ch;
                }
                if ((b & TESTPATTERN_CONTINUATIONBYTE) == PATTERN_CONTINUATIONBYTE) {
                    fwd = (fwd << 6) | (b & MASK_PRECIOUSBITS_CONTINUATIONBYTE);
                    more--;
                    if (more == 0) {
                        res.append((char) fwd);
                    }
                } else if ((b & TESTPATTERN_1BYTE) == PATTERN_1BYTE) {
                    res.append((char) b);
                } else if ((b & TESTPATTERN_2BYTES) == PATTERN_2BYTES) {
                    fwd = b & MASK_PRECIOUSBITS_HIGHESTBYTE_2BYTES;
                    more = 1;
                } else if ((b & TESTPATTERN_3BYTES) == PATTERN_3BYTES) {
                    fwd = b & MASK_PRECIOUSBITS_HIGHESTBYTE_3BYTES;
                    more = 2;
                } else if ((b & TESTPATTERN_4BYTES) == PATTERN_4BYTES) {
                    fwd = b & MASK_PRECIOUSBITS_HIGHESTBYTE_4BYTES;
                    more = 3;
                } else if ((b & TESTPATTERN_5BYTES) == PATTERN_5BYTES) {
                    fwd = b & MASK_PRECIOUSBITS_HIGHESTBYTE_5BYTES;
                    more = 4;
                } else {
                    fwd = b & MASK_PRECIOUSBITS_HIGHESTBYTE_MOREBYTES;
                    more = 5;
                }
            }
            return res.toString();
        } else {
            return null;
        }
    }

    public static RdbmsDataProviderOperations getDataProviderOperations(ApplicationContext context) {
        // return (RdbmsDataProviderOperations) context.getBean("dpOperations");
        return (RdbmsDataProviderOperations) context.getBean("dataProviderOperations");
    }

    public static String removeAccents(String text) {
        return text == null ? null : Normalizer.normalize(text, Normalizer.Form.NFD).replaceAll("\\p{InCombiningDiacriticalMarks}+", "");
    }
}
