package hu.kozadat.search.base.preprocessor;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.GregorianCalendar;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * User: codex
 */
@Component("dateRangePreprocessor")
public class DateRangePreprocessor extends QueryPreprocessor {

    private static final Pattern dateRangePattern = Pattern.compile("(^|[\\s()])(-?[0-9]{1,4})[.]{2}(-?[0-9]{1,4})(\\b|\\B)");

    private DatePrecision datePrecision;

    public DatePrecision getDatePrecision() {
        return datePrecision;
    }

    @Autowired
    public void setDatePrecision(DatePrecision datePrecision) {
        this.datePrecision = datePrecision;
    }

    private String formatStartYear(int year) {
        GregorianCalendar cal = new GregorianCalendar(year, 0, 1);
        Date event = cal.getTime();

        if (datePrecision != null)
            event = datePrecision.adjustDateDownwards(event);

        return df.format(event);
    }

    private String formatEndYear(int year) {
        GregorianCalendar cal = new GregorianCalendar(year, 11, 31);
        Date event = cal.getTime();

        if (datePrecision != null)
            event = datePrecision.adjustDateUpwards(event);

        return df.format(event);
    }

    public QueryPreprocessorResult preprocess(String segment, boolean appendFieldName) {
        QueryPreprocessorResult res = new QueryPreprocessorResult();
        res.setOriginalQuery(segment);
        Matcher dateRangeMatcher = dateRangePattern.matcher(segment);
        StringBuffer sb = new StringBuffer();

        String fieldStr = appendFieldName ? "date:" : "";

        while (dateRangeMatcher.find()) {
            String startStr = dateRangeMatcher.group(1);
            String startYearStr = dateRangeMatcher.group(2);
            String endYearStr = dateRangeMatcher.group(3);
            int startYear = 0;
            int endYear = 0;
            boolean validRange = true;

            try {
                startYear = Integer.parseInt(startYearStr);
                endYear = Integer.parseInt(endYearStr);

                if (endYear < startYear)
                    validRange = false;
            } catch (NumberFormatException nex) {
                validRange = false;
            }

            String modifiedDateRange;
            if (validRange)
                modifiedDateRange = startStr + fieldStr + "[" + formatStartYear(startYear) + " TO " + formatEndYear(endYear) + "]";
            else
                modifiedDateRange = startStr;

            dateRangeMatcher.appendReplacement(sb, modifiedDateRange);
            res.setQueryModified(true);
        }
        dateRangeMatcher.appendTail(sb);

        res.setFilteredQuery(sb.toString());
        return res;
    }
}
