package hu.kozadat.search.persistence;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.MessageSource;
import org.springframework.stereotype.Repository;

import java.util.Locale;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * A kereso adatbazis cache bean-je.
 *
 * @author Csaba Horvath, Scriptum Inc.
 * @version $Id$
 */
@Repository("searchCache")
public class SearchCache {

    private static final Logger LOGGER = LoggerFactory.getLogger(SearchCache.class);

    public static final String ELEMENT_PREFIX = "org.dublincore.dc.elements.";
    public static final String QUALIFIER_PREFIX = "org.dublincore.dc.element_qualifiers.";

    private static final Pattern p = Pattern.compile("</?(.*?)(s.*?)?>");

    private MessageSource dublinCoreElements;

    @Autowired
    @Qualifier("dublinCoreElements")
    public void setDublinCoreElements(MessageSource dublinCoreElements) {
        this.dublinCoreElements = dublinCoreElements;
    }

    public synchronized String getHungarianElementName(String elementName) {
        return dublinCoreElements.getMessage(ELEMENT_PREFIX + elementName, new Object[]{}, Locale.getDefault());
    }

    public synchronized String getHungarianQualifierName(String qualifierName) {
        return dublinCoreElements.getMessage(QUALIFIER_PREFIX + qualifierName, new Object[]{}, Locale.getDefault());
    }

    public String getLabelTranslation(String label) {
        return dublinCoreElements.getMessage(ELEMENT_PREFIX + label, new Object[]{}, Locale.getDefault());
    }

    public static String stripHtmlTags(String text) {
        Matcher m = p.matcher(text);

        StringBuffer out = new StringBuffer();
        int lastPos = 0;
        while (m.find()) {
            m.group(1);
            out.append(text.substring(lastPos, m.start()));
            lastPos = m.end();
        }

        if (lastPos > 0) {
            out.append(text.substring(lastPos));
            return out.toString().trim();
        } else {
            return text;
        }
    }

}
