package hu.kozadat.search.persistence.impl;

import hu.kozadat.search.persistence.DataProviderOperations;
import hu.kozadat.search.persistence.UserOperations;
import hu.kozadat.search.persistence.core.RdbmsDataProviderOperations;
import hu.kozadat.search.service.SearchUIConfiguration;
import hu.scriptum.einfoszab.api.EInfoSzabOperationException;
import hu.scriptum.einfoszab.datamodel.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * Created with IntelliJ IDEA.
 * User: tnagy
 * Date: 2012.05.24.
 * Time: 12:21
 * To change this template use File | Settings | File Templates.
 */
@Repository("dataProviderOperations")
public class DataProviderOperationsImpl implements DataProviderOperations, InitializingBean {

    private static final Logger LOGGER = LoggerFactory.getLogger(DataProviderOperationsImpl.class);

    private JdbcTemplate jdbcTemplate;
    private RdbmsDataProviderOperations rdbmsDataProviderOperations;

    private UserOperations userOperations;

    private SearchUIConfiguration searchUIConfiguration;

    @Autowired
    public void setJdbcTemplate(JdbcTemplate jdbcTemplate) {
        this.jdbcTemplate = jdbcTemplate;
        this.rdbmsDataProviderOperations = new RdbmsDataProviderOperations();
        this.rdbmsDataProviderOperations.setJdbcTemplate(this.jdbcTemplate);
    }

    @Autowired
    public void setUserOperations(UserOperations userOperations) {
        this.userOperations = userOperations;
    }

    @Autowired
    public void setSearchUIConfiguration(SearchUIConfiguration searchUIConfiguration) {
        this.searchUIConfiguration = searchUIConfiguration;
    }

    public void afterPropertiesSet() throws Exception {

        // todo tnagy remove later
        rdbmsDataProviderOperations.setUserOperations(((UserOperationsImpl) userOperations).getRdbmsUserOperations());

        // todo tnagy read from config
//        property name="defaultSchemaIds" value="http://schema.einfoszab.hu/kozadat/kozadat_altalanos/"
//        property name="modificationUrl" value="http://kozadat-infinea.dyndns.org/indexer/search/changeSignalForm"
//        property name="needIpFilter" value="false"
//        rdbmsDataProviderOperations.setHarvesterIpList(harvesterIpList);
//        rdbmsDataProviderOperations.setModificationUrl(modificationUrl);
//        rdbmsDataProviderOperations.setNeedIpFilter(needIpFilter);
    }

    public List<DataProvider> getDataProvidersInSettlement(long settlementId) {
        return rdbmsDataProviderOperations.getDataProvidersInSettlement(settlementId);
    }

    public List<DataProvider> getDataProvidersInSubRegion(long subRegionId) {
        return rdbmsDataProviderOperations.getDataProvidersInSubRegion(subRegionId);
    }

    public Map<Long, Long> getDataProviderCountInSubRegionsByCounty(long countyId) {
        return rdbmsDataProviderOperations.getDataProviderCountInSubRegionsByCounty(countyId);
    }

    public Map<Long, Long> getDataProviderCountInSubRegionsByRegion(long regionId) {
        return rdbmsDataProviderOperations.getDataProviderCountInSubRegionsByRegion(regionId);
    }

    public List<DataProvider> getDataProvidersInCounty(long countyId) {
        return rdbmsDataProviderOperations.getDataProvidersInCounty(countyId);
    }

    public SubRegion getSubRegion(long subRegionId) {
        return rdbmsDataProviderOperations.getSubRegion(subRegionId);
    }

    public List<SubRegion> getSubRegionsInCounty(long countyId) {
        return rdbmsDataProviderOperations.getSubRegionsInCounty(countyId);
    }

    public List<SubRegion> getSubRegionsInRegion(long regionId) {
        return rdbmsDataProviderOperations.getSubRegionsInRegion(regionId);
    }

    public Region getRegion(Long regionId) {
        return rdbmsDataProviderOperations.getRegion(regionId);
    }

    public List<Region> getRegions() {
        return rdbmsDataProviderOperations.getRegions();
    }

    public County getCounty(Long countyId) {
        return rdbmsDataProviderOperations.getCounty(countyId);
    }

    public void setDefaultProviderPriority(int defaultProviderPriority) {
        rdbmsDataProviderOperations.setDefaultProviderPriority(defaultProviderPriority);
    }

    public void addDataProvider2(DataProvider dataProvider) throws EInfoSzabOperationException {
        rdbmsDataProviderOperations.addDataProvider2(dataProvider);
    }

    public void addDPPhones(DataProvider dp) {
        rdbmsDataProviderOperations.addDPPhones(dp);
    }

    public int deleteDPPhones(long dpId) {
        return rdbmsDataProviderOperations.deleteDPPhones(dpId);
    }

    public void addInstitutionTypes(DataProvider dp) {
        rdbmsDataProviderOperations.addInstitutionTypes(dp);
    }

    public int deleteInstitutionTypes(long dpId) {
        return rdbmsDataProviderOperations.deleteInstitutionTypes(dpId);
    }

    public List<Integer> getInstitutionTypeIds(List<String> types) {
        return rdbmsDataProviderOperations.getInstitutionTypeIds(types);
    }

    public synchronized void addDataProvider(DataProvider dataProvider) throws EInfoSzabOperationException {
        rdbmsDataProviderOperations.addDataProvider(dataProvider);
    }

    public DataProvider getDataProvider(long id) throws EInfoSzabOperationException {
        return rdbmsDataProviderOperations.getDataProvider(id);
    }

    public void deleteDataProvider(DataProvider dataProvider) throws EInfoSzabOperationException {
        rdbmsDataProviderOperations.deleteDataProvider(dataProvider);
    }

    public void saveDataProviderChanges(DataProvider provider) throws EInfoSzabOperationException {
        rdbmsDataProviderOperations.saveDataProviderChanges(provider);
    }

    public void sendActivityRequest(DataProvider provider) throws EInfoSzabOperationException {
        rdbmsDataProviderOperations.sendActivityRequest(provider);
    }

    public void sendDeleteRequest(DataProvider provider) throws EInfoSzabOperationException {
        rdbmsDataProviderOperations.sendDeleteRequest(provider);
    }

    public String getOaiRepositoryConfigurationXml(DataProvider provider) throws EInfoSzabOperationException {
        return rdbmsDataProviderOperations.getOaiRepositoryConfigurationXml(provider);
    }

    public List getSupportedSchemas() {
        return rdbmsDataProviderOperations.getSupportedSchemas();
    }

    public List getDefaultSchemas() {
        return rdbmsDataProviderOperations.getDefaultSchemas();
    }

    public void setDefaultSchemaIds(Set defaultSchemaIds) {
        rdbmsDataProviderOperations.setDefaultSchemaIds(defaultSchemaIds);
    }

    public List getUserTypes() {
        return rdbmsDataProviderOperations.getUserTypes();
    }

    public int getActiveRecordCount(DataProvider provider) {
        return rdbmsDataProviderOperations.getActiveRecordCount(provider);
    }

    public void addUserType(String type) throws EInfoSzabOperationException {
        rdbmsDataProviderOperations.addUserType(type);
    }

    public void removeUserType(long id) throws EInfoSzabOperationException {
        rdbmsDataProviderOperations.removeUserType(id);
    }

    public long getDataProviderId(String name) throws EInfoSzabOperationException {
        return rdbmsDataProviderOperations.getDataProviderId(name);
    }

    public String getDataProviderName(long dataProviderId) {
        return rdbmsDataProviderOperations.getDataProviderName(dataProviderId);
    }

    public String getDataProviderOaiId(long dataProviderId) {
        return rdbmsDataProviderOperations.getDataProviderOaiId(dataProviderId);
    }

    public List<InstitutionType> getInstitutionTypes() {
        return rdbmsDataProviderOperations.getInstitutionTypes();
    }

    public List getDPPhones(long dataProviderId) {
        return rdbmsDataProviderOperations.getDPPhones(dataProviderId);
    }

    public List getUserPhones(String loginName) {
        return rdbmsDataProviderOperations.getUserPhones(loginName);
    }

    public List getUserEmails(String loginName) {
        return rdbmsDataProviderOperations.getUserEmails(loginName);
    }

    public List getInstitutionTypes(long dataProviderId) {
        return rdbmsDataProviderOperations.getInstitutionTypes(dataProviderId);
    }

    public int updateInstitutionData(DataProvider dp) {
        return rdbmsDataProviderOperations.updateInstitutionData(dp);
    }

    public int updateOaiData(DataProvider dp) {
        return rdbmsDataProviderOperations.updateOaiData(dp);
    }

    public int updateHarvestInterval(long dataProviderId, int harvestInterval) {
        return rdbmsDataProviderOperations.updateHarvestInterval(dataProviderId, harvestInterval);
    }

    public String getHarvestURL(long dataproviderId) {
        return rdbmsDataProviderOperations.getHarvestURL(dataproviderId);
    }

    public Address getAddress(long dataProviderId) {
        return rdbmsDataProviderOperations.getAddress(dataProviderId);
    }

    public ZipCode getZipCode(long zipCodeSettlementId) {
        return rdbmsDataProviderOperations.getZipCode(zipCodeSettlementId);
    }

    public List<ZipCode> getZipCodes() {
        return rdbmsDataProviderOperations.getZipCodes();
    }

    public List<ZipCode> getZipCodesByPattern(String pattern) {
        return rdbmsDataProviderOperations.getZipCodesByPattern(pattern);
    }

    public String toString() {
        return rdbmsDataProviderOperations.toString();
    }

}
