/**
 * Created by IntelliJ IDEA.
 * User: tnagy
 * Date: 2010.02.11.
 * Time: 10:20:54
 */
package hu.kozadat.search.persistence.recommendedsearch;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.jdbc.core.JdbcOperations;
import org.springframework.jdbc.core.RowMapper;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

public class RdbmsDictionarySource implements DictionarySource {

    private JdbcOperations jdbcTemplate;

    private static final Logger LOGGER = LoggerFactory.getLogger(RdbmsDictionarySource.class);

    private static final String TABLE_DICTIONARY = "dictionary";
    private static final String COLUMN_DICTIONARY_ID = "id";
    private static final String COLUMN_DICTIONARY_WORD = "word";
    private static final String COLUMN_DICTIONARY_SUBSTITUTE = "substitute";
    private static final String COLUMN_DICTIONARY_TYPE = "type";

    private static final String QUERY_DICTIONARY_WORDS_BY_TYPE =
            "SELECT " +
                    COLUMN_DICTIONARY_ID + ", " +
                    COLUMN_DICTIONARY_WORD + ", " +
                    COLUMN_DICTIONARY_SUBSTITUTE + ", " +
                    COLUMN_DICTIONARY_TYPE + " " +
                    "FROM " +
                    TABLE_DICTIONARY + " " +
                    "WHERE " +
                    COLUMN_DICTIONARY_TYPE + "=?";

    public void setJdbcTemplate(JdbcOperations jdbcTemplate) {
        this.jdbcTemplate = jdbcTemplate;
    }

    public Map<String, String> readCriticalWords() {
        return readWords(WordType.Critical.getId());
    }

    public Map<String, String> readSubstituteWords() {
        return readWords(WordType.Substitute.getId());
    }

    public Map<String, String> readWords(long type) {
        if (LOGGER.isDebugEnabled()) {
            LOGGER.debug("readDictionary() CALLED, params: type=" + type);
        }
        boolean skipRead = true;
        if (skipRead) {
            return new TreeMap();
        }
        long started = System.currentTimeMillis();
        List result = jdbcTemplate.query(QUERY_DICTIONARY_WORDS_BY_TYPE, new Object[]{type}, new RowMapper() {
            public Object mapRow(ResultSet resultSet, int i) throws SQLException {
                String word = resultSet.getString(COLUMN_DICTIONARY_WORD);
                String substitute = resultSet.getString(COLUMN_DICTIONARY_SUBSTITUTE);

                return new Word(word, substitute);
            }
        });
        Map<String, String> words = new TreeMap<String, String>();
        for (Object o : result) {
            if (o instanceof Word) {
                Word wordObject = (Word) o;
                words.put(wordObject.getWord(), wordObject.getSubstitute());
            }
        }
        if (LOGGER.isDebugEnabled()) {
            LOGGER.debug("readDictionary() FINISHED, words=" + words.size() + " in " + (System.currentTimeMillis() - started) + " ms");
        }
        return words;
    }
}
