/**
 * Created by IntelliJ IDEA.
 * User: tnagy
 * Date: 2010.01.19.
 * Time: 10:56:03
 */
package hu.kozadat.search.presentation;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;

/**
 * Google Maps IFrame src attribute creator and google maps iframe request parameter container for EInfoSzab
 */
public class GMIFrameSrcParam {

    private static final Logger LOGGER = LoggerFactory.getLogger(GMIFrameSrcParam.class);

    public static final String GOOGLE_MAPS_IFRAME_BASE_URL = "http://maps.google.com/maps";

    public static final String PARAM_FORM_TYPE = "f";
    public static final String PARAM_SOURCE_TYPE = "source";
    public static final String PARAM_HOST_LANGUAGE = "hl";
    public static final String PARAM_GEOCODE = "geocode";
    public static final String PARAM_OUTPUT_ENCODING = "oe";
    public static final String PARAM_INPUT_ENCODING = "ie";
    public static final String PARAM_ZOOM = "z";
    public static final String PARAM_OUTPUT_TYPE = "output";
    public static final String PARAM_QUERY = "q";
    public static final String PARAM_CENTER_LATITUDE_LONGITUDE = "ll";
    public static final String PARAM_SEARCH_LATITUDE_LONGITUDE = "sll";
    public static final String PARAM_SPAN = "spn";
    public static final String PARAM_SCREEN_SPAN = "sspn";

    public static final String FORM_TYPE_EIFOSZAB = "q";
    public static final String SOURCE_TYPE_EINFOSZAB = "s_q";
    public static final String HOST_LANGUAGE_EINFOSZAB = "hu";
    public static final String GEOCODE_EINFOSZAB = "";
    public static final String OUTPUT_ENCODING_EINFOSZAB = "UTF8";
    public static final String INPUT_ENCODING_EINFOSZAB = "UTF8";
    // member 'zoom' has no EInfoSzab-default value
    public static final String OUTPUT_TYPE_EINFOSZAB = "embed";
    // member 'query' has no EInfoSzab-default value
    // member 'll' has no EInfoSzab-default value
    public static final double SEARCH_LATITUDE_EINFOSZAB = 37.0625;
    public static final double SEARCH_LONGITUDE_EINFOSZAB = -95.677068;
    public static final String SEARCH_LATITUDE_LONGITUDE_EINFOSZAB = Double.toString(SEARCH_LATITUDE_EINFOSZAB) + "," + Double.toString(SEARCH_LONGITUDE_EINFOSZAB);
    public static final double SPAN_LATITUDE_INITIAL_EINFOSZAB = 150.0;
    public static final double SPAN_LONGITUDE_INITIAL_EINFOSZAB = 300.0;
    // member 'spn' has no EInfoSzab-default value
    public static final double SCREEN_SPAN_LATITUDE_EINFOSZAB = 53.609468;
    public static final double SCREEN_SPAN_LONGITUDE_EINFOSZAB = 90.263672;
    public static final String SCREEN_SPAN_EINFOSZAB = Double.toString(SCREEN_SPAN_LATITUDE_EINFOSZAB) + "," + Double.toString(SCREEN_SPAN_LONGITUDE_EINFOSZAB);

    public static int DEFAULT_ZOOM = 6;

    private String formType;
    private String sourceType;
    private String hostLanguage;
    private String geocode;
    private String outputEncoding;
    private String inputEncoding;
    private String zoom;
    private String outputType;
    private String query;
    private String latitudeLongitude;
    private double latitude;
    private double longitude;
    private String searchLatitudeLongitude;
    private String span;
    private String screenSpan;

    public static GMIFrameSrcParam getEInfoSzabDefault() {
        return new GMIFrameSrcParam(FORM_TYPE_EIFOSZAB, SOURCE_TYPE_EINFOSZAB, HOST_LANGUAGE_EINFOSZAB, GEOCODE_EINFOSZAB, OUTPUT_ENCODING_EINFOSZAB, INPUT_ENCODING_EINFOSZAB, null, OUTPUT_TYPE_EINFOSZAB, null, null, SEARCH_LATITUDE_LONGITUDE_EINFOSZAB, null, SCREEN_SPAN_EINFOSZAB);
    }

    public GMIFrameSrcParam() {
    }

    public GMIFrameSrcParam(String formType, String sourceType, String hostLanguage, String geocode, String outputEncoding, String inputEncoding, String zoom, String outputType, String query, String latitudeLongitude, String searchLatitudeLongitude, String span, String screenSpan) {
        this.formType = formType;
        this.sourceType = sourceType;
        this.hostLanguage = hostLanguage;
        this.geocode = geocode;
        this.outputEncoding = outputEncoding;
        this.inputEncoding = inputEncoding;
        this.zoom = zoom;
        this.outputType = outputType;
        this.query = query;
        this.latitudeLongitude = latitudeLongitude;
        this.searchLatitudeLongitude = searchLatitudeLongitude;
        this.span = span;
        this.screenSpan = screenSpan;
    }

    public String getFormType() {
        return formType;
    }

    public void setFormType(String formType) {
        this.formType = formType;
    }

    public String getSourceType() {
        return sourceType;
    }

    public void setSourceType(String sourceType) {
        this.sourceType = sourceType;
    }

    public String getHostLanguage() {
        return hostLanguage;
    }

    public void setHostLanguage(String hostLanguage) {
        this.hostLanguage = hostLanguage;
    }

    public String getOutputEncoding() {
        return outputEncoding;
    }

    public void setOutputEncoding(String outputEncoding) {
        this.outputEncoding = outputEncoding;
    }

    public String getInputEncoding() {
        return inputEncoding;
    }

    public void setInputEncoding(String inputEncoding) {
        this.inputEncoding = inputEncoding;
    }

    public String getZoom() {
        return zoom;
    }

    public void setZoom(String zoom) {

        int zoomLevel;
        try {
            zoomLevel = Integer.parseInt(zoom);
        } catch (NumberFormatException e) {
            LOGGER.warn("zoom=" + zoom + ", " + e.getMessage(), e);
            // set default
            zoomLevel = DEFAULT_ZOOM;
        }

        setZoom(zoomLevel);
    }

    public void setZoom(int zoom) {
        double spnLatitude = SPAN_LATITUDE_INITIAL_EINFOSZAB / Math.pow(2, zoom - 1);
        double spnLongitude = SPAN_LONGITUDE_INITIAL_EINFOSZAB / Math.pow(2, zoom - 1);
        this.span = Double.toString(spnLatitude) + "," + Double.toString(spnLongitude);

        this.zoom = Integer.toString(zoom);
    }

    public String getOutputType() {
        return outputType;
    }

    public void setOutputType(String outputType) {
        this.outputType = outputType;
    }

    public String getQuery() {
        return query;
    }

    public void setQuery(String query) {
        this.query = query;
    }

    public double getLatitude() {
        return latitude;
    }

    public double getLongitude() {
        return longitude;
    }

    public String getLatitudeLongitude() {
        return latitudeLongitude;
    }

    public void setLatitudeLongitude(Double latitude, Double longitude) {
        if (latitude == null || longitude == null) {
            this.latitudeLongitude = "";
        } else {
            this.latitude = latitude;
            this.longitude = longitude;
            this.latitudeLongitude = Double.toString(latitude) + "," + Double.toString(longitude);
        }
    }

    public String getSearchLatitudeLongitude() {
        return searchLatitudeLongitude;
    }

    public void setSearchLatitudeLongitude(String searchLatitudeLongitude) {
        this.searchLatitudeLongitude = searchLatitudeLongitude;
    }

    public String getSpan() {
        return span;
    }

    /**
     * setZoom() takes care of span
     *
     * @param span the span as String (nothing set, use setZoom())
     */
    public void setSpan(String span) {
        // this.span = span;
    }

    public String getScreenSpan() {
        return screenSpan;
    }

    public void setScreenSpan(String screenSpan) {
        this.screenSpan = screenSpan;
    }

    public String getSrcValue() {
        StringBuilder value = new StringBuilder(GOOGLE_MAPS_IFRAME_BASE_URL);

        value.append("?").append(PARAM_FORM_TYPE).append("=").append(formType);
        value.append("&amp;").append(PARAM_SOURCE_TYPE).append("=").append(sourceType);
        value.append("&amp;").append(PARAM_HOST_LANGUAGE).append("=").append(hostLanguage);
        value.append("&amp;").append(PARAM_GEOCODE).append("=").append(geocode);
        String encodedQuery = query;
        try {
            encodedQuery = URLEncoder.encode(query, "UTF-8");
        } catch (UnsupportedEncodingException e) {
            LOGGER.error(e.getMessage(), e);
        }
        value.append("&amp;").append(PARAM_QUERY).append("=").append(encodedQuery);
        value.append("&amp;").append(PARAM_SEARCH_LATITUDE_LONGITUDE).append("=").append(searchLatitudeLongitude);
        value.append("&amp;").append(PARAM_SCREEN_SPAN).append("=").append(screenSpan);
        value.append("&amp;").append(PARAM_INPUT_ENCODING).append("=").append(inputEncoding);
        value.append("&amp;").append(PARAM_OUTPUT_ENCODING).append("=").append(outputEncoding);
        value.append("&amp;").append(PARAM_CENTER_LATITUDE_LONGITUDE).append("=").append(latitudeLongitude);
        value.append("&amp;").append(PARAM_SPAN).append("=").append(span);
        value.append("&amp;").append(PARAM_ZOOM).append("=").append(zoom);
        value.append("&amp;").append(PARAM_OUTPUT_TYPE).append("=").append(outputType);

        return value.toString();
    }

    public void setNecessaryValues(String query, Double latitude, Double longitude, int zoom) {
        setNecessaryValues(query, latitude, longitude, zoom, false);
    }

    public void setNecessaryValues(String query, Double latitude, Double longitude, int zoom, boolean useMapCenterCorrection) {
        setQuery(query);
        setZoom(zoom);
        if (!useMapCenterCorrection) {
            setLatitudeLongitude(latitude, longitude);
        } else {
            double latitudeCorrection = SPAN_LATITUDE_INITIAL_EINFOSZAB / Math.pow(2, zoom);
            double longitudeCorrection = SPAN_LONGITUDE_INITIAL_EINFOSZAB / Math.pow(2, zoom) * -1;
            latitude = latitude + latitudeCorrection;
            longitude = longitude + longitudeCorrection;
            setLatitudeLongitude(latitude, longitude);
        }
    }

}
